/* $Id: suid-kgi.c,v 1.6 1999/06/27 16:01:15 marcus Exp $
******************************************************************************

   Suid-GI "kernel" driver

   Copyright (C) 1998	Andreas Beck	[becka@ggi-project.org]
  
   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (the "Software"),
   to deal in the Software without restriction, including without limitation
   the rights to use, copy, modify, merge, publish, distribute, sublicense,
   and/or sell copies of the Software, and to permit persons to whom the
   Software is furnished to do so, subject to the following conditions:

   The above copyright notice and this permission notice shall be included in
   all copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
   THE AUTHOR(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
   IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
   CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

******************************************************************************
*/

#define MAX_NR_CONSOLES 1

#define DEBUG 1
#define DEBUG_LEVEL 4

#include <kgi/maintainers.h>
#define	MAINTAINER	Andreas_Beck
#define DRIVER_NAME	"suid"
#define DRIVER_REV	"$Revision: 1.6 $"
/*#define KGI_OSTYPE	linuxsuid */

#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include <kgi/module.h>

extern struct kgi_font *kgi_default_font(kgi_modereq *mode);
extern kgi_clut *kgi_default_clut(kgi_modereq *mode);
extern int kgi_register_display(struct kgi_display *dpy, int id);
extern int kgi_unregister_display(struct kgi_display *dpy, struct kgi_display *alias);


#undef printk
int printk(char *format,...)
{
	va_list ap;
	va_start(ap, format);
	return vfprintf( stderr, format, ap);
}


#if DEBUG_LEVEL > 0
static void printk_mode (struct kgi_mode *mode, char *msg)
{
	#define M(X)	mode->X
	#define R(X)	mode->request.X
	#define T(X)	M(X.width), M(X.blankstart), M(X.syncstart),\
			M(X.syncend), M(X.blankend), M(X.total), M(X.polarity)

	printk(msg);
	printk("mode = {\n{\n/* frames    */  %i,\n/* visible */\t{ %i, %i},\n",
			R(frames), R(visible.x),R(visible.y));
	printk("/* virtual   */  { %i, %i},\n",R(virt.x),R(virt.y));
	printk("/* size [mm] */  { %i, %i},\n",R(size.x),R(size.y));
	printk("/* graphmode */  0x%x,\n", R(graphtype));
	printk("/* dppgrid   */  { %i, %i}},\n", R(dpp.x),R(dpp.y));
	printk("/* x timing  */  { %i, %i, %i, %i, %i, %i, %i},\n", T(x));
	printk("/* y timing  */  { %i, %i, %i, %i, %i, %i, %i},\n", T(y));
	printk("/* magnify   */  { %i, %i},\n", M(magnify.x), M(magnify.y));
	printk("/* flags     */  %x,\n", M(flags));
	printk("/* dclk      */  %i,\n", M(dclk));
	printk("/* clk       */  { %i, %i},\n", M(clk.mul), M(clk.div));
	printk("/* lclk      */  { %i, %i}\n}\n", M(lclk.mul), M(lclk.div));

	#undef M
	#undef R
	#undef T
}
#endif /* #if DEBUG_LEVEL > 0 */

/* This stuff should be picked up from the KGI driver */
#define CONFIG_CARD_MANUFACT    "Manufacturer"
#define CONFIG_CARD_MODEL       "Model"
#define CONFIG_CARD_MODES       GM_ALL
#define CONFIG_CARD_MAX_X       0x7FFF
#define CONFIG_CARD_MAX_Y       0x7FFF

static struct kgi_card card = 
{
	CONFIG_CARD_MANUFACT,			/* Card manufacturer	*/
	CONFIG_CARD_MODEL,			/* Card model		*/
	CONFIG_CARD_MODES,			/* Allowed modes	*/
	0,
	{ CONFIG_CARD_MAX_X, CONFIG_CARD_MAX_Y },/* User limits		*/

	NULL,	/* card component info structs, these are not known yet	*/
	NULL,	/* and get filled in during inititalization		*/
	NULL,
	NULL,
	NULL,

	{	/* The mode to set when resetting the card		*/
		{ 1,				/* need one frame	*/
		  { 80, 25 }, { 80, 25 },	/* visible/virtual	*/
		  { 0, 0 },			/* size in mm		*/
		  KGIGT_TEXT16,			/* mode			*/
		  { 9, 16 }			/* text size		*/
		},

		{ 720, 720, 765, 873, 882, 900,  0 },	/* horiz.timing	*/
		{ 400, 407, 412, 414, 442, 449,  1 },	/* vert. timing	*/
		{ 1, 1 },			/* no magnification	*/
		0,	/* !!! check */		/* special capabilities	*/
		28321000,			/* dot clock in Hz	*/
		{ 1, 1 },			/* clock rate to program*/
		{ 1, 1 },			/* transfer rate to dac	*/

		0,
        }
};

#define	kgim_set_mode	kgim_chipset_set_mode

static int kgim_check_mode(struct kgi_display *dpy, struct kgi_mode *mode, 
			   enum kgi_tm_cmd cmd)
{
	int cnt = 10;

	if (cmd == CMD_PROPOSE) {

		memset(&(mode->request) + 1, 0, 
		       sizeof(*mode) - sizeof(mode->request));
	}

	while (cnt-- && !(cmd == CMD_READY)) {

		register int err;
		DEBUG1("check_mode:cmd=%i; cnt=%i", cmd, cnt);

		if ((err = kgim_clock_check_mode (dpy, mode, &cmd))) {

			return err;
		}
		
		DEBUG1("check_mode: clock ok.");

		if ((err = kgim_ramdac_check_mode(dpy, mode, &cmd))) {

			return err;
		}
		
		DEBUG1("check_mode: ramdac ok.");

		if ((err = kgim_chipset_check_mode(dpy, mode, &cmd))) {

			return err;
		}
		
		DEBUG1("check_mode: chipset ok.");
 
		if (cmd == CMD_LOWER || cmd == CMD_RAISE) {

			if ((err = kgim_clock_check_mode(dpy, mode, &cmd))) 
			{

				return err;
			}
			DEBUG1("2nd clock check ok.");
		}

		if ((err = kgim_accel_check_mode(dpy, mode, &cmd))) {

			return err;
		}
		
		DEBUG1("check_mode: graphics ok.");
 
		if ((err = kgim_monitor_check_mode(dpy, mode, &cmd))) {

			return err;
		}
		
		DEBUG1("check_mode: monitor ok.");

		#if DEBUG_LEVEL > 0
			printk_mode(mode, "mode:\n");
		#endif
	}

	#if DEBUG_LEVEL > 0
	printk_mode(mode, "mode ok:\n");
	#endif
 
	return (++cnt) ? EOK : -E(DRIVER, UNKNOWN);
}

static void kgim_reset(struct kgi_display *dpy)
{
	DEBUG1("kgim_reset: About to set mode.");
	kgim_chipset_set_mode(dpy, &(dpy->card->preset));

	DEBUG1("kgim_reset: About to set font.");
	if (dpy->set_font) {

		struct kgi_font *font;

		DEBUG1("kgim_reset: About to get default font.");
		font = kgi_default_font(&(dpy->card->preset.request));
		DEBUG1("kgim_reset: About to set font via %p.",dpy->set_font);
		(dpy->set_font)(dpy, font, 0, font->info->positions);
	}

	DEBUG1("kgim_reset: About to set clut.");
	if (dpy->set_clut) {

		kgi_clut *clut;

		DEBUG1("kgim_reset: About to get default clut.");
		clut = kgi_default_clut(&(dpy->card->preset.request));
		DEBUG1("kgim_reset: About to set clut.");
		(dpy->set_clut)(dpy, clut, 0, clut->size);
	}

	/*	frame and origin are set to zero and splitline is set to
	**	dpy->card->reset->request.visible.y by the chipset driver
	**	during dpy->set_mode. Thus no need to reset them again.
	*/
	DEBUG1("kgim_reset: returning.");
}

/*	module loading
*/

kgi_uint pcibus = -1, pcidev = -1, pcifn = 0;
kgi_uint pcibase0 = 0, pcibase1 = 0, pcibase2 = 0, pcibase3 = 0, pcibase4 = 0;
kgi_uint dacmhz = -1;
char	*chipset = NULL;
struct kgim_options_pci options_pci;

kgi_uint law_base = 0, display = 0;

#ifdef MODULE_PARM
#if MAX_NR_DISPLAYS != 32
	#warning MAX_NR_DISPLAYS changed, update this code
#endif
MODULE_PARM(display, "0-" __MODULE_STRING(32) "i");

MODULE_PARM(law_base, "1-" __MODULE_STRING(4294967295) "i");

MODULE_PARM(pcibus, "0-" __MODULE_STRING(255) "i");
MODULE_PARM(pcidev, "0-" __MODULE_STRING(255) "i");
MODULE_PARM(pcibase0, "1-" __MODULE_STRING(429496295) "i");
MODULE_PARM(pcibase1, "1-" __MODULE_STRING(429496295) "i");
MODULE_PARM(pcibase2, "1-" __MODULE_STRING(429496295) "i");
MODULE_PARM(pcibase3, "1-" __MODULE_STRING(429496295) "i");
MODULE_PARM(pcibase4, "1-" __MODULE_STRING(429496295) "i");

MODULE_PARM(dacmhz, "1-" __MODULE_STRING(255) "i");

MODULE_PARM(chipset, "s");
#endif

struct kgim_options_misc options_misc;
struct kgim_options_chipset options_chipset;

static struct kgim_options options =
{
	&options_misc,
	&options_pci,
	&options_chipset
};

static inline void get_options(void)
{
	if ((pcibus != -1) && (pcidev != -1)) {

		options_pci.dev = PCICFG_VADDR(pcibus, pcidev, pcifn);
		options_pci.base0 = pcibase0;
		options_pci.base1 = pcibase1;
		options_pci.base2 = pcibase2;
		options_pci.base3 = pcibase3;
		options_pci.base4 = pcibase4;

	} else {

		memset(&options_pci, 0, sizeof(options_pci));
		options_pci.dev = PCICFG_NULL;
	}

	options_misc.law_base = law_base;
	options_misc.display = display;

	options_chipset.chipset = chipset;
};

static struct kgim_private module_private =
{
	&options,
	NULL,
};

static kgi_modereq kgim_default_mode;

struct kgi_display my_display = 
{
	-1,				/* id number			*/
	0,				/* refcnt			*/
	0xFFFFFFFF,			/* conflict_text		*/
	0xFFFFFFFF,			/* conflict_graph		*/
	0xFFFFFFFF,			/* conflict_setup		*/
	NULL,				/* currently mapped device	*/

	&module_private,		/* driver_data			*/

					/* these are set by the driver	*/

	NULL,				/* enable(struct kgi_display *)	*/
	NULL,				/* disable(struct kgi_display *)*/
	kgim_reset,			/* reset(struct kgi_display *)	*/

	{				/* ioctl methods		*/
		NULL,
		kgim_monitor_command,
		kgim_ramdac_command,
		kgim_clock_command,
		kgim_chipset_command,
		kgim_accel_command,
		NULL,
		NULL
	},
	&card,				/* hardware info		*/
	kgim_check_mode,		/* check_mode()			*/
	kgim_set_mode,			/* set_mode()	-> chipset	*/

	NULL,				/* set_font()	-> chipset	*/
	NULL,				/* set_clut()	-> dac		*/
	NULL,				/* set_split()	-> chipset	*/
	NULL,				/* set_frame()	-> chipset	*/
	NULL,				/* set_origin()	-> chipset	*/
	NULL,				/* set_pointer() -> dac/chipset	*/

	NULL, NULL, NULL,		/* { show, hide, undo }_mouse	*/
	NULL, NULL, NULL,		/* { show, hide, undo }_cursor	*/

	{ },				/* current mode			*/
	&kgim_default_mode,		/* default mode of display	*/
	{ }				/* framebuffer of current frame	*/
};

int suidkgi_init_module(void)
{
	/*	NOTE: The various ?_init() functions have to printk the reason
	**	for failing themself -- just in case they fail.
	*/
	int err;

	get_options();

	/*	The chipset provides I/O functions for all other parts, thus we
	**	have to init it first.
	*/
	if ((err = kgim_chipset_init(&card, &my_display))) {

		return err;
	}

	/*	Now that the chipset is ok, we try to init the monitor driver.
	**	if this fails, we have to deinitializee the chipset.
	*/
	if ((err = kgim_monitor_init(&card, &my_display))) {

		kgim_chipset_done(&my_display);
		return err;
	}

	/*	Now we go for the DAC. If it fails, we have to deinitialize
	**	the subsystems already fired up.
	*/
	if ((err = kgim_ramdac_init(&card, &my_display))) {

		kgim_monitor_done(&my_display);
		kgim_chipset_done(&my_display);
		return -EIO;
	}

	/*	Now the clock driver. If this fails, ...
	*/
	if ((err = kgim_clock_init(&card, &my_display))) {

		kgim_ramdac_done(&my_display);
		kgim_monitor_done(&my_display);
		kgim_chipset_done(&my_display);
		return -EIO;
	}

	/*	Last is the accel driver. If this fails, ...
	*/
	if ((err = kgim_accel_init(&card, &my_display))) {

		kgim_clock_done(&my_display);
		kgim_ramdac_done(&my_display);
		kgim_monitor_done(&my_display);
		kgim_chipset_done(&my_display);
		return -EIO;
	}

	/*	We require that the card can be reset properly, thus
	**	the preset mode must be accepted by the drivers.
	**	Just in case ... (Ever heard of Murphys Law? |-)
	*/
	if (kgim_check_mode(&my_display, &card.preset, CMD_CHECK)) {
	
		ERROR("check_mode() failed with preset mode.");
		kgim_accel_done(&my_display);
		kgim_clock_done(&my_display);
		kgim_ramdac_done(&my_display);
		kgim_monitor_done(&my_display);
		kgim_chipset_done(&my_display);
		return -EIO;
	}

	/*	Now set the initial display mode and other state. We know
	**	the mode check can't fail because if it does, we should never
	**	have gotten here.
	*/
	memcpy(my_display.defmode, &card.preset.request, sizeof(kgi_modereq));
	memcpy(&my_display.mode.request, my_display.defmode, sizeof(kgi_modereq));
	DEBUG1("About to check mode.");
	kgim_check_mode(&my_display, &my_display.mode, CMD_PROPOSE);
	DEBUG1("About to set mode.");
	kgim_chipset_set_mode(&my_display, &my_display.mode);
	DEBUG1("About to reset mode.");
	kgim_reset(&my_display);

	/*	Ok. Hardware is up and running now, so we can try to register
	**	it. <display> holds the boot display to replace or -1 if the
	**	display has not been registered. If registration fails, we
	**	reset the hardware and shutdown properly.
	*/
	DEBUG1("About to register display mode.");
	if ((err = kgi_register_display(&my_display, display))) {

		ERROR("failed to register as display %i", display);
		kgim_reset(&my_display);

		kgim_accel_done(&my_display);
		kgim_clock_done(&my_display);
		kgim_ramdac_done(&my_display);
		kgim_monitor_done(&my_display);
		kgim_chipset_done(&my_display);
		return -EIO;
	}

	/*	And now.... FLY BABY, FLY !
	*/
	DEBUG1("%s %s driver loaded as display %i.",
	       card.manufact, card.model, my_display.id);
	return EOK;
}

void suidkgi_cleanup_module (void)
{

	kgim_reset(&my_display);

	kgim_accel_done(&my_display);
	kgim_clock_done(&my_display);
	kgim_ramdac_done(&my_display);
	kgim_monitor_done(&my_display);
	kgim_chipset_done(&my_display);

	DEBUG1("%s %s driver removed.", card.manufact, card.model);
}
