/* $Id: kgimon.h,v 1.6 1999/02/05 00:59:51 marcus Exp $
******************************************************************************

   Misc routines to deal with KGI monitor databases
   
   Copyright (C) 1998-1999 Marcus Sundberg	[marcus@ggi-project.org]
   
   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (the "Software"),
   to deal in the Software without restriction, including without limitation
   the rights to use, copy, modify, merge, publish, distribute, sublicense,
   and/or sell copies of the Software, and to permit persons to whom the
   Software is furnished to do so, subject to the following conditions:

   The above copyright notice and this permission notice shall be included in
   all copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
   THE AUTHOR(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
   IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
   CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

******************************************************************************
*/

#include <stdio.h>
#include <kgi/kgi.h>
#include <kgi/kgi_commands.h>

#include "config.h"

#if defined (HAVE_UNISTD_H) && defined(HAVE_FCNTL_H) && defined (HAVE_SYS_IOCTL_H)
#include <unistd.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#define HAVE_SET_MONSTUFF
#endif

#define END_OF_FILE	-1
#define PARSE_ERROR	-2
#define NOFILE_ERROR	-3
#define SETPARAM_ERROR	-4

#define COMMENT_CHAR	'#'

#define MAX_NAMELEN	63
#define MAX_LINE	4096


int
skip_white(FILE *fp);

int
skip_line(FILE *fp);

int
skip_comments(FILE *fp);

char *
str_stripwhite(char *str);

kgi_graphmask
parse_graphmask(char *str);

enum kgi_flags
parse_kgiflags(char *str);

enum kgi_display_type
parse_displaytype(char *str);

enum kgi_sync
parse_synctype(char *str);

kgiu16
parse_polarity(char *str);

void
fprint_coord(FILE *fp, char *name, kgi_coord *coord);

int
fprint_monitor(FILE *fp, struct kgi_monitor *mon);

int
read_dbentry(FILE *fp, struct kgi_monitor *monp, struct kgi_preset **pres);


#ifdef HAVE_SET_MONSTUFF
int
get_curtiming(int fd, struct kgi_preset *pres);

int
set_monitor(int fd, struct kgi_monitor *mon);

int
set_preset(int fd, struct kgi_preset *pres);

int
clr_presets(int fd);

int
set_presets(int fd, struct kgi_preset *pres);

int
set_htiming(int fd, struct kgi_timing *htim);

int
set_vtiming(int fd, struct kgi_timing *vtim);

int
clr_timings(int fd);

int
set_timings(int fd, struct kgi_timing *htim, struct kgi_timing *vtim);

int
set_monfromfile(FILE *fp);
#endif
